"""Entity."""

from pygleif.address import Address
from pygleif.const import (
    ATTR_ADDRESS_TYPE_HQ,
    ATTR_ADDRESS_TYPE_LEGAL,
    ATTR_ENTITY,
    ATTR_ENTITY_LEGAL_FORM,
    ATTR_ENTITY_LEGAL_NAME,
    ATTR_ENTITY_REGISTERED_AS,
    ATTR_ID,
    ATTR_JURISDICTION,
    ATTR_NAME,
    ATTR_STATUS,
    LEGAL_FORMS,
)


class GLEIFEntity:
    def __init__(self, entity):
        self._entity = entity

    @property
    def raw(self):
        """Return raw data."""
        return self._entity.raw[ATTR_ENTITY]

    @property
    def registration_authority_entity_id(self):
        """
        Some entities return the register entity id,
        but other do not. Unsure if this is a bug or
        inconsistently registered data.
        """
        return self.raw.get(ATTR_ENTITY_REGISTERED_AS)

    @property
    def entity_status(self):
        """Return entity status."""
        return self.raw.get(ATTR_STATUS)

    @property
    def legal_form(self):
        """In some cases, the legal form is stored in the JSON-data.
        In other cases, an ELF-code, consisting of mix of exactly
        four letters and numbers are stored. This ELF-code
        can be looked up in a registry where a code maps to
        a organizational type. ELF-codes are not unique,
        it can reoccur under different names in different
        countries"""
        legal_form_id = self.raw.get(ATTR_ENTITY_LEGAL_FORM).get(ATTR_ID)

        if (
            self.legal_jurisdiction in LEGAL_FORMS
            and legal_form_id in LEGAL_FORMS[self.legal_jurisdiction]
        ):
            return LEGAL_FORMS[self.legal_jurisdiction][legal_form_id]

        return f"ELF code: {legal_form_id}"

    @property
    def legal_jurisdiction(self):
        """Return legal jurisdiction."""
        return self.raw.get(ATTR_JURISDICTION)

    @property
    def legal_name(self):
        """Return legal name."""
        return self.raw.get(ATTR_ENTITY_LEGAL_NAME).get(ATTR_NAME)

    @property
    def headquarters_address(self):
        return Address(self, ATTR_ADDRESS_TYPE_HQ)

    @property
    def legal_address(self):
        return Address(self, ATTR_ADDRESS_TYPE_LEGAL)



from .const import (
    ATTR_LEI,
    URL_API,
)
import urllib.request as url
import json

from .entity import GLEIFEntity
from .registration import GLEIFRegistration


class GLEIF:
    """Parse JSON from GLEIF registry. Supports v1 of API."""

    def __init__(self, lei_code):
        self.lei_code = lei_code

    @property
    def json_data(self):
        return url.urlopen(URL_API + self.lei_code)

    @property
    def lei_exists(self):
        try:
            self.raw
            return False
        except IndexError:
            return True

    @property
    def raw(self):
        return json.loads(self.json_data.read().decode("UTF-8"))["data"]["attributes"]

    @property
    def lei(self):
        return self.raw[ATTR_LEI]

    @property
    def entity(self):
        return GLEIFEntity(self)

    @property
    def registration(self):
        return GLEIFRegistration(self)



URL_API = "https://api.gleif.org/api/v1/lei-records/"
URL_LEVEL2_CONCAT_FILES = (
    "https://leidata.gleif.org/api/v1/concatenated-files/rr/%date%/zip"
)
URL_SEARCH = "https://api.gleif.org/api/v1/lei-records?filter%5Bfulltext%5D="
URL_DIRECT_CHILD = (
    "https://api.gleif.org/api/v1/lei-records/{}/direct-" "child-relationships"
)

ATTR_LEI = "lei"
ATTR_ENTITY_REGISTERED_AS = "registeredAs"
ATTR_JURISDICTION = "jurisdiction"
ATTR_ENTITY_LEGAL_NAME = "legalName"
ATTR_NAME = "name"
ATTR_STATUS = "status"
ATTR_ENTITY_LEGAL_FORM = "legalForm"
ATTR_ID = "id"


# RegistrationAuthorityEntityID
ATTR_ENTITY_ENTITY_STATUS = "EntityStatus"
ATTR_ENTITY_LEGAL_FORM_CODE = "EntityLegalFormCode"

ATTR_INITIAL_REGISTRATION_DATE = "initialRegistrationDate"
ATTR_LAST_UPDATE_DATE = "lastUpdateDate"
ATTR_MANAGING_LOU = "managingLou"
ATTR_NEXT_RENEWAL_DATE = "nextRenewalDate"
ATTR_VALIDATION_SOURCES = "corroborationLevel"

ATTR_ADDRESS_TYPE_HQ = "headquartersAddress"
ATTR_ADDRESS_TYPE_LEGAL = "legalAddress"
ATTR_ADDRESS_CITY = "city"
ATTR_ADDRESS_COUNTRY = "country"
ATTR_ADDRESS_LINE1 = "addressLines"
ATTR_ADDRESS_POSTAL_CODE = "postalCode"
ATTR_ADDRESS_REGION = "region"

ATTR_REGISTER = "@register"
ATTR_REGISTRATION = "registration"
ATTR_ENTITY = "entity"

# Allowed attributes for a record
ALLOW_ATTR_REGISTRATION_STATUS = ["ISSUED", "LAPSED"]

# The ELF codes are unique per country, not globally.
# Hence the nested dict.
LEGAL_FORMS = {
    "NO": {
        "326Y": "Ansvarlig selskap",
        "3C7U": "Stiftelse",
        "3L58": "Kommunalt foretak",
        "4ZRR": "Norskregistrert utenlandsk foretak",
        "50TD": "Statsforetak",
        "5ZTZ": "Fylkeskommunalt foretak",
        "8S9H": "Gjensidig forsikringsselskap",
        "9DI1": "Boligbyggelag",
        "AEV1": "Partsrederi",
        "BJ65": "Pensjonskasse",
        "CF5L": "Ansvarlig selskap, delt ansvar",
        "DRPL": "Europeisk økonomisk foretaksgruppe",
        "EXD7": "Enkeltpersonforetak",
        "FSBD": "Forening/lag/innretning (eller bare forening?)",
        "GYY6": "Annen juridisk person",
        "IQGE": "Allmennaksjeselskap",
        "K5P8": "Samvirkeforetak",
        "LJJW": "Verdipapirfond",
        "M9IQ": "Kommandittselskap",
        "O0EU": "Borettslag",
        "O7LB": "Europeisk samvirkeforetak",
        "PB3V": "Konkursbo",
        "Q0Q1": "Selskap med begrenset ansvar",
        "R71C": "Sparebank",
        "V06W": "Europeisk selskap",
        "YI42": "Aksjeselskap",
        "YTMC": "Interkommunalt selskap",
        "ZQ0Q": "Eierseksjonssameie",
    },
    "SE": {
        "1TN0": "Ideell förening (som bedriver näringsverksamhet)",
        "2UAX": "Trossamfund (som bedriver näringsverksamhet)",
        "381R": "Medlemsbank",
        "54P7": "Europabolag",
        "9YIP": "Europakooperativ",
        "AZTO": "Sambruksförening",
        "BEAY": "Handelsbolag",
        "BYQJ": "Bankaktiebolag",
        "C61P": "Ekonomisk förening",
        "CX05": "Kommanditbolag",
        "G04R": "Sparbank",
        "M0Y0": "Försäkringsförening",
        "O1QI": "Ömsesidigt försäkringsbolag",
        "OJ9I": "Försäkringsaktiebolag",
        "PDQ0": "Kooperativ hyresrättsförening",
        "RLJO": "Europeisk ekonomisk intressegruppering",
        "RYFP": "Europeisk gruppering för territoriellt samarbete",
        "SSOM": "Bostadsrättsförening",
        "WZDB": "Bostadsförening",
        "XJHM": "Aktiebolag",
    },
}



"""Search."""
import json
from urllib import request as url

from bs4 import BeautifulSoup

from pygleif.const import ALLOW_ATTR_REGISTRATION_STATUS, URL_DIRECT_CHILD, URL_SEARCH


class Search:
    """Class to use the search form of the GLEIF web site."""

    def __init__(self, orgnr=None):
        """Init class."""
        # Allow searching using organisation number
        self.orgnr = orgnr

    @property
    def json_data(self):
        """Get raw data from the service."""
        return url.urlopen(URL_SEARCH + url.quote(self.orgnr))

    @property
    def raw(self):
        """Return parsed json."""
        return json.loads(self.json_data.read().decode("UTF-8"))

    @property
    def valid_record(self):
        """Loop through data to find a valid record. Return first valid."""
        for d in self.raw["data"]:

            # We're not very greedy here, but it seems some records have
            # lapsed even through the issuer is active
            if (
                d["attributes"]["registration"]["status"]
                in ALLOW_ATTR_REGISTRATION_STATUS
            ):
                return d["attributes"]

    @property
    def lei(self):
        """Return the LEI code."""
        try:
            return self.valid_record["lei"]
        except (IndexError, TypeError):
            return None


class GLEIFParseRelationshipRecord:
    def __init__(self, record_xml):
        self.record_xml = record_xml

    @property
    def raw(self):
        return BeautifulSoup(self.record_xml, "xml")


class DirectChild:
    def __init__(self, lei=""):
        """Init class."""
        # Allow searching using organisation number
        self.lei = lei

    @property
    def json_data(self):
        """Get raw data from the service."""
        return url.urlopen(URL_DIRECT_CHILD.format(self.lei))

    @property
    def raw(self):
        """Return parsed json."""
        return json.loads(self.json_data.read().decode("UTF-8"))

    @property
    def valid_child_records(self):
        child_lei = list()

        """Loop through data to find a valid record. Return list of LEI."""
        for d in self.raw["data"]:

            # We're not very greedy here, but it seems some records have
            # lapsed even through the issuer is active
            if d["attributes"]["relationship"]["status"] in ["ACTIVE"]:
                child_lei.append(d["attributes"]["relationship"]["startNode"]["id"])

        return child_lei



#!/usr/bin/env python3
"""Set up pygleif."""

from pathlib import Path

from setuptools import find_packages, setup

PROJECT_DIR = Path(__file__).parent.resolve()
README_FILE = PROJECT_DIR / "README.md"
LONG_DESCRIPTION = README_FILE.read_text(encoding="utf-8")

VERSION = (PROJECT_DIR / "pygleif" / "VERSION").read_text().strip()

GITHUB_URL = "https://github.com/ggravlingen/pygleif"
DOWNLOAD_URL = f"{GITHUB_URL}/archive/{VERSION}.zip"

PACKAGES = find_packages(exclude=["tests", "tests.*"])

setup(
    name="pygleif",
    packages=PACKAGES,
    python_requires=">=3.8",
    version=VERSION,
    long_description=LONG_DESCRIPTION,
    author='ggravlingen',
    author_email="no@email.com",
    long_description_content_type="text/markdown",
    url=GITHUB_URL,
    include_package_data=True,
    license="MIT",
    keywords='lei-code lei api gleif leicode',
    download_url=DOWNLOAD_URL,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Home Automation",
    ],
)



"""Registration data."""
from dateutil import parser

from pygleif.const import (
    ATTR_INITIAL_REGISTRATION_DATE,
    ATTR_LAST_UPDATE_DATE,
    ATTR_MANAGING_LOU,
    ATTR_NEXT_RENEWAL_DATE,
    ATTR_REGISTRATION,
    ATTR_STATUS,
    ATTR_VALIDATION_SOURCES,
)


class GLEIFRegistration:
    def __init__(self, registration):
        self._registration = registration

    @property
    def raw(self):
        return self._registration.raw.get(ATTR_REGISTRATION)

    @property
    def initial_registration_date(self):
        if ATTR_INITIAL_REGISTRATION_DATE in self.raw:
            return parser.parse(self.raw.get(ATTR_INITIAL_REGISTRATION_DATE))

    @property
    def last_update_date(self):
        if ATTR_LAST_UPDATE_DATE in self.raw:
            return parser.parse(self.raw.get(ATTR_LAST_UPDATE_DATE))

    @property
    def managing_lou(self):
        if ATTR_MANAGING_LOU in self.raw:
            return self.raw[ATTR_MANAGING_LOU]

    @property
    def next_renewal_date(self):
        if ATTR_NEXT_RENEWAL_DATE in self.raw:
            return parser.parse(self.raw[ATTR_NEXT_RENEWAL_DATE])

    @property
    def registration_status(self):
        if ATTR_STATUS in self.raw:
            return self.raw[ATTR_STATUS]

    @property
    def validation_sources(self):
        if ATTR_VALIDATION_SOURCES in self.raw:
            return self.raw[ATTR_VALIDATION_SOURCES]



"""Address."""
from pygleif.const import (
    ATTR_ADDRESS_CITY,
    ATTR_ADDRESS_COUNTRY,
    ATTR_ADDRESS_LINE1,
    ATTR_ADDRESS_POSTAL_CODE,
    ATTR_ADDRESS_REGION,
)


class Address:
    def __init__(self, address, address_type):
        self._address = address
        self.address_type = address_type

    @property
    def raw(self):
        return self._address.raw.get(self.address_type)

    @property
    def city(self):
        return self.raw.get(ATTR_ADDRESS_CITY, None)

    @property
    def country(self):
        return self.raw.get(ATTR_ADDRESS_COUNTRY, None)

    @property
    def line1(self):
        address_lines = self.raw.get(ATTR_ADDRESS_LINE1, None)
        if address_lines:
            return address_lines[0]

    @property
    def postal_code(self):
        return self.raw.get(ATTR_ADDRESS_POSTAL_CODE, None)

    @property
    def region(self):
        return self.raw.get(ATTR_ADDRESS_REGION, None)
